import torch
import transformers
# import transformers.models.llama.modeling_llama

max_len = 8192
class ScaledRotaryEmbedding(torch.nn.Module):
    def __init__(self, dim, max_position_embeddings=2048, base=10000, device=None):
        super().__init__()
        inv_freq = 1.0 / (base ** (torch.arange(0, dim, 2).float().to(device) / dim))
        self.register_buffer("inv_freq", inv_freq)
        
        # extend
        max_position_embeddings = 8192

        # Build here to make `torch.jit.trace` work.
        self.max_seq_len_cached = max_position_embeddings
        t = torch.arange(
            self.max_seq_len_cached,
            device=self.inv_freq.device,
            dtype=self.inv_freq.dtype,
        )

        self.scale = 1 / 4
        t *= self.scale

        freqs = torch.einsum("i,j->ij", t, self.inv_freq)
        # Different from paper, but it uses a different permutation in order to obtain the same calculation
        emb = torch.cat((freqs, freqs), dim=-1)
        self.register_buffer(
            "cos_cached", emb.cos()[None, None, :, :], persistent=False
        )
        self.register_buffer(
            "sin_cached", emb.sin()[None, None, :, :], persistent=False
        )

    def forward(self, x, seq_len=None):
        # x: [bs, num_attention_heads, seq_len, head_size]
        # This `if` block is unlikely to be run after we build sin/cos in `__init__`. Keep the logic here just in case.
        if seq_len > self.max_seq_len_cached:
            self.max_seq_len_cached = seq_len
            t = torch.arange(
                self.max_seq_len_cached, device=x.device, dtype=self.inv_freq.dtype
            )
            t *= self.scale
            freqs = torch.einsum("i,j->ij", t, self.inv_freq)
            # Different from paper, but it uses a different permutation in order to obtain the same calculation
            emb = torch.cat((freqs, freqs), dim=-1).to(x.device)
            self.register_buffer(
                "cos_cached", emb.cos()[None, None, :, :], persistent=False
            )
            self.register_buffer(
                "sin_cached", emb.sin()[None, None, :, :], persistent=False
            )
        return (
            self.cos_cached[:, :, :seq_len, ...].to(dtype=x.dtype),
            self.sin_cached[:, :, :seq_len, ...].to(dtype=x.dtype),
        )


# def replace_llama_rope_with_scaled_rope():
#     transformers.models.llama.modeling_llama.LlamaRotaryEmbedding = (
#         ScaledRotaryEmbedding
#     )

def help():
    print("""
    用法示例：
        def replace_llama_rope_with_scaled_rope():
            transformers.models.llama.modeling_llama.LlamaRotaryEmbedding = (
                ScaledRotaryEmbedding
            )
        replace_llama_rope_with_scaled_rope()
    """)


from nltk.translate.bleu_score import sentence_bleu
import jieba
from rouge import Rouge

import warnings
warnings.filterwarnings("ignore")

# BLEU
def bleu_score(true_answer : str, generate_answer : str) -> float:
    # true_anser : 标准答案，str 类型
    # generate_answer : 模型生成答案，str 类型
    true_answers = list(jieba.cut(true_answer))
    generate_answers = list(jieba.cut(generate_answer))
    bleu_score = sentence_bleu(true_answers, generate_answers)
    return bleu_score

def rouge_score(true_answer : str, generate_answer : str) -> float:
    # true_anser : 标准答案，str 类型
    # generate_answer : 模型生成答案，str 类型
    rouge = Rouge()
    rouge_score = rouge.get_scores(generate_answer, true_answer)
    return rouge_score



from metrics import *
from scaled_rope import *

help_text = """
上传步骤：
1.python setup.py check
2.python setup.py sdist build
3.twine upload dist/*
"""

def help():
    print(help_text)



import argparse

class Argument:
    def __init__(self):
        self.args = argparse.ArgumentParser()

    def add(self, console_arg):
        self.args.add_argument(console_arg)



# 多维评估
# 使用大模型评估生成文本质量
LLM_EVAL_PROMPT = '''
        你是一个模型回答评估员。
        接下来，我将给你一个问题、对应的知识片段以及模型根据知识片段对问题的回答。
        请你依次评估以下维度模型回答的表现，分别给出打分：

        ① 知识查找正确性。评估系统给定的知识片段是否能够对问题做出回答。如果知识片段不能做出回答，打分为0；如果知识片段可以做出回答，打分为1。

        ② 回答一致性。评估系统的回答是否针对用户问题展开，是否有偏题、错误理解题意的情况，打分分值在0~1之间，0为完全偏题，1为完全切题。

        ③ 回答幻觉比例。该维度需要综合系统回答与查找到的知识片段，评估系统的回答是否出现幻觉，打分分值在0~1之间,0为全部是模型幻觉，1为没有任何幻觉。

        ④ 回答正确性。该维度评估系统回答是否正确，是否充分解答了用户问题，打分分值在0~1之间，0为完全不正确，1为完全正确。

        ⑤ 逻辑性。该维度评估系统回答是否逻辑连贯，是否出现前后冲突、逻辑混乱的情况。打分分值在0~1之间，0为逻辑完全混乱，1为完全没有逻辑问题。

        ⑥ 通顺性。该维度评估系统回答是否通顺、合乎语法。打分分值在0~1之间，0为语句完全不通顺，1为语句完全通顺没有任何语法问题。

        ⑦ 智能性。该维度评估系统回答是否拟人化、智能化，是否能充分让用户混淆人工回答与智能回答。打分分值在0~1之间，0为非常明显的模型回答，1为与人工回答高度一致。

        你应该是比较严苛的评估员，很少给出满分的高评估。
        用户问题：
        ```
        {}
        ```
        待评估的回答：
        ```
        {}
        ```
        给定的知识片段：
        ```
        {}
        ```
        你应该返回给我一个可直接解析的 Python 字典，字典的键是如上维度，值是每一个维度对应的评估打分。
        不要输出任何其他内容。
    '''